@@ -22,7 +22,7 @@ from weakref import ref as weakref
 from datetime import datetime
 
 from logbook.helpers import to_safe_json, parse_iso8601, cached_property, \
-     F, PY2, u, string_types, iteritems, integer_types
+     PY2, u, string_types, iteritems, integer_types
 try:
     from logbook._speedups import group_reflected_property, \
          ContextStackManager, StackedObject
@@ -372,6 +372,8 @@ class LogRecord(object):
         #: optional exception information.  If set, this is a tuple in the
         #: form ``(exc_type, exc_value, tb)`` as returned by
         #: :func:`sys.exc_info`.
+        #: This parameter can also be ``True``, which would cause the exception info tuple
+        #: to be fetched for you.
         self.exc_info = exc_info
         #: optional extra information as dictionary.  This is the place
         #: where custom log processors can attach custom context sensitive
@@ -405,6 +407,8 @@ class LogRecord(object):
         self.time = _datetime_factory()
         if self.frame is None and Flags.get_flag('introspection', True):
             self.frame = sys._getframe(1)
+        if self.exc_info is True:
+            self.exc_info = sys.exc_info()
 
     def pull_information(self):
         """A helper function that pulls all frame-related information into
@@ -470,6 +474,7 @@ class LogRecord(object):
         self._channel = None
         if isinstance(self.time, string_types):
             self.time = parse_iso8601(self.time)
+        self.extra = ExtraDict(self.extra)
         return self
 
     @cached_property
@@ -479,11 +484,11 @@ class LogRecord(object):
             return self.msg
         try:
             try:
-                return F(self.msg).format(*self.args, **self.kwargs)
+                return self.msg.format(*self.args, **self.kwargs)
             except UnicodeDecodeError:
                 # Assume an unicode message but mixed-up args
                 msg = self.msg.encode('utf-8', 'replace')
-                return F(msg).format(*self.args, **self.kwargs)
+                return msg.format(*self.args, **self.kwargs)
             except (UnicodeEncodeError, AttributeError):
                 # we catch AttributeError since if msg is bytes, it won't have the 'format' method
                 if sys.exc_info()[0] is AttributeError and (PY2 or not isinstance(self.msg, bytes)):
@@ -494,7 +499,7 @@ class LogRecord(object):
                 # but this codepath is unlikely (if the message is a constant
                 # string in the caller's source file)
                 msg = self.msg.decode('utf-8', 'replace')
-                return F(msg).format(*self.args, **self.kwargs)
+                return msg.format(*self.args, **self.kwargs)
 
         except Exception:
             # this obviously will not give a proper error message if the
@@ -502,10 +507,10 @@ class LogRecord(object):
             # access to the frame.  But there is not much we can do about
             # that.
             e = sys.exc_info()[1]
-            errormsg = F('Could not format message with provided '
-                         'arguments: {err}\n  msg={msg!r}\n  '
-                         'args={args!r} \n  kwargs={kwargs!r}.\n'
-                         'Happened in file {file}, line {lineno}').format(
+            errormsg = ('Could not format message with provided '
+                       'arguments: {err}\n  msg={msg!r}\n  '
+                       'args={args!r} \n  kwargs={kwargs!r}.\n'
+                       'Happened in file {file}, line {lineno}').format(
                 err=e, msg=self.msg, args=self.args,
                 kwargs=self.kwargs, file=self.filename,
                 lineno=self.lineno
@@ -627,7 +632,10 @@ class LogRecord(object):
         if self.exc_info is not None:
             val = self.exc_info[1]
             try:
-                return u(str(val))
+                if PY2:
+                    return unicode(val)
+                else:
+                    return str(val)
             except UnicodeError:
                 return str(val).decode('utf-8', 'replace')
 
@@ -729,7 +737,9 @@ class LoggerMixin(object):
 
     def catch_exceptions(self, *args, **kwargs):
         """A context manager that catches exceptions and calls
-        :meth:`exception` for exceptions caught that way.  Example::
+        :meth:`exception` for exceptions caught that way.  Example:
+
+        .. code-block:: python
 
             with logger.catch_exceptions():
                 execute_code_that_might_fail()
@@ -836,6 +846,15 @@ class RecordDispatcher(object):
             if not handler.should_handle(record):
                 continue
 
+            # a filter can still veto the handling of the record.  This
+            # however is already operating on an initialized and processed
+            # record.  The impact is that filters are slower than the
+            # handler's should_handle function in case there is no default
+            # handler that would handle the record (delayed init).
+            if handler.filter is not None \
+               and not handler.filter(record, handler):
+                continue
+
             # if this is a blackhole handler, don't even try to
             # do further processing, stop right away.  Technically
             # speaking this is not 100% correct because if the handler
@@ -857,15 +876,6 @@ class RecordDispatcher(object):
                 self.process_record(record)
                 record_initialized = True
 
-            # a filter can still veto the handling of the record.  This
-            # however is already operating on an initialized and processed
-            # record.  The impact is that filters are slower than the
-            # handler's should_handle function in case there is no default
-            # handler that would handle the record (delayed init).
-            if handler.filter is not None \
-               and not handler.filter(record, handler):
-                continue
-
             # handle the record.  If the record was handled and
             # the record is not bubbling we can abort now.
             if handler.handle(record) and not handler.bubble:
