@@ -28,15 +28,15 @@ from logbook.base import CRITICAL, ERROR, WARNING, NOTICE, INFO, DEBUG, \
      NOTSET, level_name_property, _missing, lookup_level, \
      Flags, ContextObject, ContextStackManager
 from logbook.helpers import rename, b, _is_text_stream, is_unicode, PY2, \
-    zip, xrange, string_types, integer_types, iteritems, reraise
+    zip, xrange, string_types, integer_types, reraise, u
 
 
 DEFAULT_FORMAT_STRING = (
-    u'[{record.time:%Y-%m-%d %H:%M}] '
-    u'{record.level_name}: {record.channel}: {record.message}'
+    u('[{record.time:%Y-%m-%d %H:%M:%S.%f}] ') +
+    u('{record.level_name}: {record.channel}: {record.message}')
 )
-SYSLOG_FORMAT_STRING = u'{record.channel}: {record.message}'
-NTLOG_FORMAT_STRING = u'''\
+SYSLOG_FORMAT_STRING = u('{record.channel}: {record.message}')
+NTLOG_FORMAT_STRING = u('''\
 Message Level: {record.level_name}
 Location: {record.filename}:{record.lineno}
 Module: {record.module}
@@ -46,10 +46,10 @@ Exact Time: {record.time:%Y-%m-%d %H:%M:%S}
 Event provided Message:
 
 {record.message}
-'''
+''')
 TEST_FORMAT_STRING = \
-u'[{record.level_name}] {record.channel}: {record.message}'
-MAIL_FORMAT_STRING = u'''\
+u('[{record.level_name}] {record.channel}: {record.message}')
+MAIL_FORMAT_STRING = u('''\
 Subject: {handler.subject}
 
 Message type:       {record.level_name}
@@ -61,14 +61,14 @@ Time:               {record.time:%Y-%m-%d %H:%M:%S}
 Message:
 
 {record.message}
-'''
-MAIL_RELATED_FORMAT_STRING = u'''\
+''')
+MAIL_RELATED_FORMAT_STRING = u('''\
 Message type:       {record.level_name}
 Location:           {record.filename}:{record.lineno}
 Module:             {record.module}
 Function:           {record.func_name}
 {record.message}
-'''
+''')
 
 SYSLOG_PORT = 514
 
@@ -125,12 +125,12 @@ class Handler(ContextObject):
         # all here goes to that handler
         handler.pop_application()
 
-    By default messages send to that handler will not go to a handler on
+    By default messages sent to that handler will not go to a handler on
     an outer level on the stack, if handled.  This can be changed by
     setting bubbling to `True`.  This setup for example would not have
     any effect::
 
-        handler = NullHandler(bubble=False)
+        handler = NullHandler(bubble=True)
         handler.push_application()
 
     Whereas this setup disables all logging for the application::
@@ -374,7 +374,7 @@ class StringFormatter(object):
         line = self.format_record(record, handler)
         exc = self.format_exception(record)
         if exc:
-            line += u'\n' + exc
+            line += u('\n') + exc
         return line
 
 
@@ -420,7 +420,7 @@ class HashingHandlerMixin(object):
         """Returns a hashlib object with the hash of the record."""
         hash = sha1()
         hash.update(('%d\x00' % record.level).encode('ascii'))
-        hash.update((record.channel or u'').encode('utf-8') + b('\x00'))
+        hash.update((record.channel or u('')).encode('utf-8') + b('\x00'))
         hash.update(record.filename.encode('utf-8') + b('\x00'))
         hash.update(b(str(record.lineno)))
         return hash
@@ -532,6 +532,12 @@ class StreamHandler(Handler, StringFormatterHandlerMixin):
         self.close()
         return Handler.__exit__(self, exc_type, exc_value, tb)
 
+    def ensure_stream_is_open(self):
+        """this method should be overriden in sub-classes to ensure that the
+        inner stream is open
+        """
+        pass
+
     def close(self):
         """The default stream handler implementation is not to close
         the wrapped stream but to flush it.
@@ -543,10 +549,10 @@ class StreamHandler(Handler, StringFormatterHandlerMixin):
         if self.stream is not None and hasattr(self.stream, 'flush'):
             self.stream.flush()
 
-    def format_and_encode(self, record):
-        """Formats the record and encodes it to the stream encoding."""
+    def encode(self, msg):
+        """Encodes the message to the stream encoding."""
         stream = self.stream
-        rv = self.format(record) + '\n'
+        rv = msg + '\n'
         if (PY2 and is_unicode(rv)) or \
                 not (PY2 or is_unicode(rv) or _is_text_stream(stream)):
             enc = self.encoding
@@ -560,9 +566,11 @@ class StreamHandler(Handler, StringFormatterHandlerMixin):
         self.stream.write(item)
 
     def emit(self, record):
+        msg = self.format(record)
         self.lock.acquire()
         try:
-            self.write(self.format_and_encode(record))
+            self.ensure_stream_is_open()
+            self.write(self.encode(msg))
             self.flush()
         finally:
             self.lock.release()
@@ -596,30 +604,31 @@ class FileHandler(StreamHandler):
         self.stream = open(self._filename, mode)
 
     def write(self, item):
-        if self.stream is None:
-            self._open()
+        self.ensure_stream_is_open()
         if not PY2 and isinstance(item, bytes):
             self.stream.buffer.write(item)
         else:
             self.stream.write(item)
 
     def close(self):
-        if self.stream is not None:
-            self.flush()
-            self.stream.close()
-            self.stream = None
+        self.lock.acquire()
+        try:
+            if self.stream is not None:
+                self.flush()
+                self.stream.close()
+                self.stream = None
+        finally:
+            self.lock.release()
 
-    def format_and_encode(self, record):
+    def encode(self, record):
         # encodes based on the stream settings, so the stream has to be
         # open at the time this function is called.
-        if self.stream is None:
-            self._open()
-        return StreamHandler.format_and_encode(self, record)
+        self.ensure_stream_is_open()
+        return StreamHandler.encode(self, record)
 
-    def emit(self, record):
+    def ensure_stream_is_open(self):
         if self.stream is None:
             self._open()
-        StreamHandler.emit(self, record)
 
 
 class MonitoringFileHandler(FileHandler):
@@ -655,12 +664,21 @@ class MonitoringFileHandler(FileHandler):
                 self._last_stat = st[stat.ST_DEV], st[stat.ST_INO]
 
     def emit(self, record):
-        last_stat = self._last_stat
-        self._query_fd()
-        if last_stat != self._last_stat:
-            self.close()
-        FileHandler.emit(self, record)
-        self._query_fd()
+        msg = self.format(record)
+        self.lock.acquire()
+        try:
+            last_stat = self._last_stat
+            self._query_fd()
+            if last_stat != self._last_stat and self.stream is not None:
+                self.flush()
+                self.stream.close()
+                self.stream = None
+            self.ensure_stream_is_open()
+            self.write(self.encode(msg))
+            self.flush()
+            self._query_fd()
+        finally:
+            self.lock.release()
 
 
 class StderrHandler(StreamHandler):
@@ -682,49 +700,6 @@ class StderrHandler(StreamHandler):
         return sys.stderr
 
 
-class RotatingFileHandlerBase(FileHandler):
-    """Baseclass for rotating file handlers.
-
-    .. versionchanged:: 0.3
-       This class was deprecated because the interface is not flexible
-       enough to implement proper file rotations.  The former builtin
-       subclasses no longer use this baseclass.
-    """
-
-    def __init__(self, *args, **kwargs):
-        from warnings import warn
-        warn(DeprecationWarning('This class is deprecated'))
-        FileHandler.__init__(self, *args, **kwargs)
-
-    def emit(self, record):
-        self.lock.acquire()
-        try:
-            msg = self.format_and_encode(record)
-            if self.should_rollover(record, msg):
-                self.perform_rollover()
-            self.write(msg)
-            self.flush()
-        finally:
-            self.lock.release()
-
-    def should_rollover(self, record, formatted_record):
-        """Called with the log record and the return value of the
-        :meth:`format_and_encode` method.  The method has then to
-        return `True` if a rollover should happen or `False`
-        otherwise.
-
-        .. versionchanged:: 0.3
-           Previously this method was called with the number of bytes
-           returned by :meth:`format_and_encode`
-        """
-        return False
-
-    def perform_rollover(self):
-        """Called if :meth:`should_rollover` returns `True` and has
-        to perform the actual rollover.
-        """
-
-
 class RotatingFileHandler(FileHandler):
     """This handler rotates based on file size.  Once the maximum size
     is reached it will reopen the file and start with an empty file
@@ -768,9 +743,10 @@ class RotatingFileHandler(FileHandler):
         self._open('w')
 
     def emit(self, record):
+        msg = self.format(record)
         self.lock.acquire()
         try:
-            msg = self.format_and_encode(record)
+            msg = self.encode(msg)
             if self.should_rollover(record, len(msg)):
                 self.perform_rollover()
             self.write(msg)
@@ -832,7 +808,8 @@ class TimedRotatingFileHandler(FileHandler):
                filename.endswith(self._fn_parts[1]):
                 files.append((os.path.getmtime(filename), filename))
         files.sort()
-        return files[:-self.backup_count + 1]
+        return files[:-self.backup_count + 1] if self.backup_count > 1\
+                else files[:]
 
     def perform_rollover(self):
         self.stream.close()
@@ -842,11 +819,12 @@ class TimedRotatingFileHandler(FileHandler):
         self._open('w')
 
     def emit(self, record):
+        msg = self.format(record)
         self.lock.acquire()
         try:
             if self.should_rollover(record):
                 self.perform_rollover()
-            self.write(self.format_and_encode(record))
+            self.write(self.encode(msg))
             self.flush()
         finally:
             self.lock.release()
@@ -1032,7 +1010,7 @@ class MailHandler(Handler, StringFormatterHandlerMixin,
     """
     default_format_string = MAIL_FORMAT_STRING
     default_related_format_string = MAIL_RELATED_FORMAT_STRING
-    default_subject = u'Server Error in Application'
+    default_subject = u('Server Error in Application')
 
     #: the maximum number of record hashes in the cache for the limiting
     #: feature.  Afterwards, record_cache_prune percent of the oldest
@@ -1156,7 +1134,7 @@ class MailHandler(Handler, StringFormatterHandlerMixin,
         """
         from smtplib import SMTP, SMTP_PORT, SMTP_SSL_PORT
         if self.server_addr is None:
-            host = 'localhost'
+            host = '127.0.0.1'
             port = self.secure and SMTP_SSL_PORT or SMTP_PORT
         else:
             host, port = self.server_addr
@@ -1353,10 +1331,10 @@ class SyslogHandler(Handler, StringFormatterHandlerMixin):
         return (facility << 3) | priority
 
     def emit(self, record):
-        prefix = u''
+        prefix = u('')
         if self.application_name is not None:
-            prefix = self.application_name + u':'
-        self.send_to_socket((u'<%d>%s%s\x00' % (
+            prefix = self.application_name + u(':')
+        self.send_to_socket((u('<%d>%s%s\x00') % (
             self.encode_priority(record),
             prefix,
             self.format(record)
@@ -1496,7 +1474,7 @@ class FingersCrossedHandler(Handler):
     handler when a severity theshold was reached with the buffer emitting.
     This now enables this logger to be properly used with the
     :class:`~logbook.MailHandler`.  You will now only get one mail for
-    each bfufered record.  However once the threshold was reached you would
+    each buffered record.  However once the threshold was reached you would
     still get a mail for each record which is why the `reset` flag was added.
 
     When set to `True`, the handler will instantly reset to the untriggered
