@@ -10,13 +10,14 @@
 """
 import re
 import os
+from collections import defaultdict
 from cgi import parse_qsl
 
-from logbook.base import RecordDispatcher, NOTSET, ERROR, NOTICE
+from logbook.base import RecordDispatcher, dispatch_record, NOTSET, ERROR, NOTICE
 from logbook.handlers import Handler, StringFormatter, \
      StringFormatterHandlerMixin, StderrHandler
 from logbook._termcolors import colorize
-from logbook.helpers import F, PY2, u, string_types, iteritems
+from logbook.helpers import PY2, string_types, iteritems, u
 
 from logbook.ticketing import TicketingHandler as DatabaseHandler
 from logbook.ticketing import BackendBase
@@ -64,7 +65,7 @@ class TwitterFormatter(StringFormatter):
 
     def format_exception(self, record):
         return u('%s: %s') % (record.exception_shortname,
-                            record.exception_message)
+                              record.exception_message)
 
     def __call__(self, record, handler):
         formatted = StringFormatter.__call__(self, record, handler)
@@ -257,9 +258,9 @@ class ExternalApplicationHandler(Handler):
                  bubble=False):
         Handler.__init__(self, level, filter, bubble)
         self.encoding = encoding
-        self._arguments = [F(arg) for arg in arguments]
+        self._arguments = list(arguments)
         if stdin_format is not None:
-            stdin_format = F(stdin_format)
+            stdin_format = stdin_format
         self._stdin_format = stdin_format
         import subprocess
         self._subprocess = subprocess
@@ -303,9 +304,8 @@ class ColorizingStreamHandlerMixin(object):
             return 'yellow'
         return 'lightgray'
 
-    def format_and_encode(self, record):
-        rv = super(ColorizingStreamHandlerMixin, self) \
-                .format_and_encode(record)
+    def format(self, record):
+        rv = super(ColorizingStreamHandlerMixin, self).format(record)
         if self.should_colorize(record):
             color = self.get_color(record)
             if color:
@@ -357,3 +357,51 @@ class ExceptionHandler(Handler, StringFormatterHandlerMixin):
         if self.should_handle(record):
             raise self.exc_type(self.format(record))
         return False
+
+class DedupHandler(Handler):
+    """A handler that deduplicates log messages.
+
+    It emits each unique log record once, along with the number of times it was emitted.
+    Example:::
+
+        with logbook.more.DedupHandler():
+            logbook.error('foo')
+            logbook.error('bar')
+            logbook.error('foo')
+
+    The expected output:::
+
+       message repeated 2 times: foo
+       message repeated 1 times: bar
+    """
+    def __init__(self, format_string='message repeated {count} times: {message}', *args, **kwargs):
+        Handler.__init__(self, bubble=False, *args, **kwargs)
+        self._format_string = format_string
+        self.clear()
+
+    def clear(self):
+        self._message_to_count = defaultdict(int)
+        self._unique_ordered_records = []
+
+    def pop_application(self):
+        Handler.pop_application(self)
+        self.flush()
+
+    def pop_thread(self):
+        Handler.pop_thread(self)
+        self.flush()
+
+    def handle(self, record):
+        if not record.message in self._message_to_count:
+            self._unique_ordered_records.append(record)
+        self._message_to_count[record.message] += 1
+        return True
+
+    def flush(self):
+        for record in self._unique_ordered_records:
+            record.message = self._format_string.format(message=record.message, count=self._message_to_count[record.message])
+            # record.dispatcher is the logger who created the message, it's sometimes supressed (by logbook.info for example)
+            dispatch = record.dispatcher.call_handlers if record.dispatcher is not None else dispatch_record
+            dispatch(record)
+        self.clear()
+
